@@ -1,7 +1,12 @@
 onlymeanESNuni = function(a=-Inf,b=Inf,mu,Sigma,lambda,tau){
-  p = length(mu)
   s = sqrt(Sigma)
   tautil = tau/sqrt(1+sum(lambda^2))
+  if(tautil< -35){
+    #print("normal aproximation")
+    Gamma  = Sigma/(1+lambda^2)
+    mub    = lambda*tau*Gamma/s
+    return(meanvarNuni(a,b,mu-mub,Gamma))
+  }
   phi    = lambda/sqrt(1+sum(lambda^2))
   eta    = invmills(tau,0,sqrt(1+lambda^2))
   Gamma  = Sigma/(1+lambda^2)
@@ -12,5 +17,5 @@ onlymeanESNuni = function(a=-Inf,b=Inf,mu,Sigma,lambda,tau){
   db     = dmvESN1(b,mu,Sigma,lambda,tau)
   F1     = mu*F0 + Sigma*(da - db) + lambda*s*eta*F0N
   muY = c(F1/F0)
-  return(list(mean = round(muY,4)))
+  return(list(mean = muY))
 }
